#include "noise.h"

void writeWhiteNoiseToTexture(float* data, int width, int height)
{
    int channels = 4; // rgba
    int length = width*height*channels;

    printf("generating %d random floats\n", length);
    for (int i = 0; i < length; i++)
    {
        data[i] = randomFloat();
    }
}

void generateBlueNoisePoints(vec2* points, int m, int n)
{
    float start = now();
    float elapsed;
    int candidatesTested = 0;

    struct Candidate
    {
        vec2 pos;
        // distance to closest point already in pattern
        float distance;
    };

    for (int i = 0; i < n; i++)
    {
        // generate a bunch of candidates
        int count = m * i;
        struct Candidate candidates[count];

        for (int j = 0; j < count; j++)
        {
            struct Candidate* c = &candidates[j];

            c->pos[0] = randomFloat();
            c->pos[1] = randomFloat();
            // the largest distance we should expect is sqrt(2),
            // so pick a maximum larger than that
            c->distance = 1.5; 

            // get the distance of the closest point already in the pattern
            for (int k = 0; k < i; k++)
            {
                float distance = glm_vec2_distance(c->pos, points[k]);
                if (distance < c->distance)
                {
                    c->distance = distance;
                }
            }

            candidatesTested += i;

            float killAfter = 10.0;
            elapsed = now() - start;
            if (elapsed > killAfter)
            {
                printf("noise generation took too long (%fs), exiting\n", elapsed);
                exit(1);
            }
        }

        // of our candidates, now determine which of them has the furthest
        // closest-distance
        int furthest = -1;
        float furthestDistance = 0.0;
        for (int j = 0; j < count; j++)
        {
            if (candidates[j].distance > furthestDistance)
            {
                furthest = j;
                furthestDistance = candidates[j].distance;
            }
        }

        glm_vec2_copy(candidates[furthest].pos, points[i]);
    }

    elapsed = now() - start;
    printf("generated %d blue noise (m=%d) points from %d candidates in %fs\n",
        n,
        m,
        candidatesTested,
        elapsed);
}

void writeBlueNoiseToTexture(float* data, int width, int height)
{
    printf("generating blue noise\n");
    int channels = 4;

    // blue noise gen parameters
    int m = 10;     // roughly controls the evenness...?
    int n = 100;    // number of points to generate

    // for each channel
    for (int channel = 0; channel < channels; channel++)
    {
        // generate a bunch of points
        vec2 points[n];
        generateBlueNoisePoints(points, m, n);

        for (int i = 0; i < width*height; i++)
        {
            int pixelIdx = i;
            int x = pixelIdx % width;
            int y = pixelIdx / width;
            float u = x / (float)width;
            float v = y / (float)height;
            vec2 pixel = {u,v};

            //float value = 0.0f;
            int componentIdx = i * channels + channel;
            for (int j = 0; j < n; j++)
            {
                float d = glm_vec2_distance(points[j], pixel);
                //value += 1.0f / (d*d);

                float r = 0.01;
                if (d < r)
                {
                    data[componentIdx] = 1.0;
                }
            }
            //data[componentIdx] = value / (float)n;
        }
    }
}

